using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using System.Linq;

namespace TwinCitiesCodeCamp.SmoothOperator.Core.Domain
{
    /// <summary>
    /// A definition of an order 
    /// </summary>
    [DataContract]
    [Serializable]
    public class Order : IOrder
    {

        public Order()
        {
            lineItems = new List<LineItem>();
        }

        /// <summary>
        /// The Customer who placed the order. 
        /// </summary>
        [DataMember]
        public Customer Customer { get; set; }

        
        private List<LineItem> lineItems; 
        /// <summary>
        /// The line items that create the order. 
        /// </summary>
        [DataMember]
        public List<LineItem> LineItems
        {
            get { return lineItems; }
            set { lineItems = value; }
        }

        /// <summary>
        /// The total of all of the <see cref="LineItems"/>
        /// </summary>
        [DataMember]
        public double OrderTotal
        {
            get
            {
                // Calculate the order total. 
                return LineItems.Sum(item => item.Price * item.Quantity);
            }
            set
            {
                // Do nothing.     
            }
        }

        /// <summary>
        /// The discount that is applied to the order. 
        /// </summary>
        [DataMember]
        public decimal Discount { get; set; }

        /// <summary>
        /// The name of the message queue that the order should attach to.
        /// </summary>
        [DataMember]
        public string MessageQueueName { get; set; }

        /// <summary>
        /// Gets the total number of items in the order. 
        /// </summary>
        [DataMember]
        public int TotalQuantity
        {
            get
            {
                return this.LineItems.Sum(item => item.Quantity); 
            }
            set
            {
                // Do nothing. 
            }
        }
    }
}