using System.Messaging;
using System.Workflow.Activities.Rules;
using Microsoft.Rules.Samples.ExternalRuleSetLibrary;
using Microsoft.Rules.Samples.ExternalRuleSetService;

namespace TwinCitiesCodeCamp.SmoothOperator.Core.Domain
{
    public class OrderRouter : IOrderRouter
    {
        private readonly IExternalRuleSetService ruleSetService;
        private readonly RuleSetInfo ruleSetInfo;

        public OrderRouter(IExternalRuleSetService ruleSetService, RuleSetInfo ruleSetInfo)
        {
            this.ruleSetService = ruleSetService;
            this.ruleSetInfo = ruleSetInfo;
        }

        public OrderRouter() : this(new CustomExternalRuleSetService(), new RuleSetInfo("PriorityProcessing"))
        {
        }

        public void Execute(IOrder order)
        {
            var ruleSet = ruleSetService.GetRuleSet(ruleSetInfo);
            var validation = new RuleValidation(order.GetType(), null);
            var execution = new RuleExecution(validation, order);
            ruleSet.Execute(execution);

            // Add the values to the queue. 
            //var message = new Message(order);
            var queue = new MessageQueue(order.MessageQueueName, QueueAccessMode.Send);
            queue.Send(order);
        }
    }
}