﻿using System;
using System.Messaging;
using System.ServiceModel;
using Microsoft.Rules.Samples.ExternalRuleSetLibrary;
using Microsoft.Rules.Samples.ExternalRuleSetService;
using TwinCitiesCodeCamp.SmoothOperator.Core.Domain;

namespace TwinCitiesCodeCamp.SmoothOperator.OrderService
{
    public class OrderService : IOrderService
    {
        public void RouteOrder(Order order)
        {
            Console.WriteLine("{0:G}: {1} Order Received.", DateTime.Now, order.Customer.MembershipLevel);
            new OrderRouter(new CustomExternalRuleSetService(), new RuleSetInfo("PriorityProcessing")).Execute(order);
        }
    }

    public class ServiceDriver
    {
        static void Main(string[] args)
        {
            using (ServiceHost host = new ServiceHost(typeof(OrderService)))
            {

                string queueName = ".\\private$\\OrderQueue";
                if (!MessageQueue.Exists(queueName))
                    MessageQueue.Create(queueName, true);

                host.Faulted += host_Faulted;
                host.Open();

                Console.WriteLine();
                Console.WriteLine("Press <ENTER> to terminate the host application");
                Console.ReadLine();


            }
        }

        static void host_Faulted(object sender, EventArgs e)
        {
            Console.WriteLine("ServiceHost faulted...will need to be restarted.");
        }
    }

}
